clear
close all
clc

%% PARAMETERS

% adding folders to path and loading data
addpath('data')
addpath('Eff_map')
load(fullfile('data','WLTP3.mat')) % loading the speed profile (WLTP3)
load("Eff_map\Efficiency.mat"); % loading the EM efficeincy map
load("Eff_map\Shaft_torque.mat"); % loading shaft torque map
load("Eff_map\Speed.mat"); % loading speed map

% setting efficiency to 100% when T = 0
Efficiency(Efficiency==0) = 100;

% velocity and acceleration profiles
vehspeed = [time_s,speed_kmh/3.6]; % speed profile in [m/s]
dt = 1; 
vehacc = (vehspeed(2:end,2)-vehspeed(1:end-1,2))./dt; %vehicle acceleration [m/s^2]
vehacc = [0; vehacc];

% simulation parameters
t_sim = 1800; % simulation time [s]
g = 9.81; % Gravity acceleration [m/s^2]
alpha = 0*pi/180; % road slope [rad]
rho = 1.25; % air density [kg/m^3] - from Onori HEV book

% battery parameters
N_s = 108; % Number of series
N_p = 1; % Number of parallels
N_b = N_s*N_p; % number of battery cells
Q_nom = 60*N_p; % nominal battery capacity [Ah]
eta_c = 0.95; % Coloumbic efficiency
load('bat_Ro_vs_SOC_data.mat'); % Ro variation with SOC - single battery cell
load('bat_Voc_vs_SOC_data.mat'); % Voc variation with SOC - single battery cell
F_Voc_s = griddedInterpolant(SOC_Voc_data(:,1),SOC_Voc_data(:,2)); % interpolated Voc data for a single cell
F_Ro_s = griddedInterpolant(SOC_Ro_data(:,1),SOC_Ro_data(:,2)); % interpolated Ro data for a single cell
SOC_vec = linspace(0,1,500);
Voc_s = F_Voc_s(SOC_vec); % vector with interpolated values of Voc_s (single cell)
Ro_s = F_Ro_s(SOC_vec); % vector with interpolated values of Ro_s (single cell)
Voc = N_b*Voc_s; 
Ro = N_b*Ro_s;

% vehicle parameters
M_veh = 1400; % vehicle mass [kg]
a = 1; % Front axle - CoG  Front axle - CoG  [m]
b = 1.3; % Rear axle - CoG  [m]
h = 0.3; % Height of CoG [m]
f_0 = 4.5*1e-3; % Static rolling coefficient [N/kN]
k = 0; % miscellaneous loss coeff [Ns/m]
r_w = 0.3; % Wheel radius [m]
C_d = 0.33; % Drag coeff
A_f = 2.15; % Frontal area [m^2]
tau_gb = 9.6; % Gear ratio
eta_gb = 0.97; % Gearbox efficiency
eta_inv = 1; % Inverter efficiency

% electric motor parameters
Speed_max = Speed(:,1);
Torque_max = Shaft_Torque(:,1);
M_eff = [Speed(:),Shaft_Torque(:),Efficiency(:)];
F = scatteredInterpolant(Speed(:),Shaft_Torque(:),Efficiency(:));

% initial condition
SOC0 = 0.8;

%% SIMULATION

open('simplified_EV_model.slx')
sim('simplified_EV_model.slx')

%% PLOTS

directory = "C:\Users\gabri\Documents\TCC - EV powertrain control\Text\images";

fig = figure();
fig.Position = [100, 100, 1000, 600];
subplot(2,1,1);
plot(T_wheel.Time,T_wheel.Data,'b','LineWidth',1.5);
grid on
title('Wheel torque');
xlabel('Time [s]');
ylabel('Torque [Nm]');
subplot(2,1,2);
plot(w_wheel.Time,w_wheel.Data,'r','LineWidth',1.5);
xlabel('Time [s]');
ylabel('Angular speed [rad/s]');
grid on 
title('Wheel angular speed');

filename = 'result_BW_T_and_w_wheel.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);


fig = figure();
fig.Position = [100, 100, 1000, 600];
subplot(2,1,1);
plot(T_EM.Time,T_EM.Data,'b','LineWidth',1.5);
grid on
title('Electric motor torque');
xlabel('Time [s]');
ylabel('Torque [Nm]');
subplot(2,1,2);
plot(w_EM_rpm.Time,w_EM_rpm.Data,'r','LineWidth',1.5);
xlabel('Time [s]');
ylabel('Angular speed [RPM]');
grid on 
title('Electric motor angular speed in RPM');

filename = 'result_BW_T_and_w_EM.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);


fig = figure();
fig.Position = [100, 100, 1000, 600];
subplot(2,1,1);
plot(P_batt.Time,P_batt.Data,'b','LineWidth',1.5);
grid on
title('Battrey power');
xlabel('Time [s]');
ylabel('Power [W]');
subplot(2,1,2);
plot(SOC.Time,SOC.Data,'r','LineWidth',1.5);
xlabel('Time [s]');
ylabel('SOC [-]');
grid on 
title('State of Charge');

filename = 'result_BW_P_b_and_SOC.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);






