clear 
close all
clc

%% PARAMETERS

% polynomial functions for battery voltage, resistance and EM efficiency
% defined in the polynimial_fits.m file, and the funcions are saved
% separately to be used directly inside this model

addpath("polynomial_fits")
addpath("data\Eff_map")
addpath("data")
addpath("tests\")
addpath("simulation_results_const_v_ref")
load(fullfile('data','WLTP3.mat')) % loading the speed profile (WLTP3)

% velocity and acceleration profiles
vehspeed = [time_s,speed_kmh/3.6]; % speed profile in [m/s]
dt = 1; 
vehacc = (vehspeed(2:end,2)-vehspeed(1:end-1,2))./dt; %vehicle acceleration [m/s^2]
vehacc = [0; vehacc];
speed_profile = vehspeed(:,2);
time_vector = vehspeed(:,1);

% simulation parameters
g = 9.81; % Gravity acceleration [m/s^2]
alpha = 0*pi/180; % road slope [rad]
rho = 1.25; % air density [kg/m^3] - from Onori HEV book

% battery parameters
N_s = 108; % Number of series
N_p = 1; % Number of parallels
N_b = N_s*N_p; % number of battery cells
Q_nom = 60*N_p; % nominal battery capacity [Ah]
eta_c = 0.95; % Coloumbic efficiency
load('bat_Ro_vs_SOC_data.mat'); % Ro variation with SOC - single battery cell
load('bat_Voc_vs_SOC_data.mat'); % Voc variation with SOC - single battery cell
F_Voc_s = griddedInterpolant(SOC_Voc_data(:,1),SOC_Voc_data(:,2)); % interpolated Voc data for a single cell
F_Ro_s = griddedInterpolant(SOC_Ro_data(:,1),SOC_Ro_data(:,2)); % interpolated Ro data for a single cell
SOC_vec = linspace(0,1,500);
Voc_s = F_Voc_s(SOC_vec); % vector with interpolated values of Voc_s (single cell)
Ro_s = F_Ro_s(SOC_vec); % vector with interpolated values of Ro_s (single cell)

% vehicle parameters
M_veh = 1400; % vehicle mass [kg]
a = 1; % Front axle - CoG  Front axle - CoG  [m]
b = 1.3; % Rear axle - CoG  [m]
h = 0.3; % Height of CoG [m]
f_0 = 4.5*1e-3; % Static rolling coefficient [N/kN]
k = 0; % miscellaneous loss coeff [Ns/m]
r_w = 0.3; % Wheel radius [m]
C_d = 0.33; % Drag coeff
A_f = 2.15; % Frontal area [m^2]
tau_gb = 9.6; % Gear ratio
eta_gb = 0.97; % Gearbox efficiency
eta_inv = 1; % Inverter efficiency

% electric motor parameters
load("Eff_map\Efficiency.mat"); % loading the EM efficeincy map
load("Eff_map\Shaft_torque.mat"); % loading shaft torque map
load("Eff_map\Speed.mat"); % loading speed map
Speed_max = Speed(:,1);
Torque_max = Shaft_Torque(:,1);
Efficiency(Efficiency == 0) = 10;
eff = Efficiency/100;
M_eff = [Speed(:),Shaft_Torque(:),Efficiency(:)];
F = scatteredInterpolant(Speed(:),Shaft_Torque(:),Efficiency(:));


% ACC and CTG Contoller parameters
default_distance = 50; % reference distance from leading vehicle [m]
tau = 0.5; % vehicle LTI model [s]
h = 4*tau; % time gap [s] (h > 2*tau)
lambda = 0.5; % CTG parameter [-]
Td = 0.01;
s = tf('s');
P = 1/(tau*s + 1);  % Vehicle simplified plant
v_set = 40;         % ACC set velocity [m/s]
time_gap  = 3;      % ACC time gap [s]                          
verr_gain = 0.1;    % ACC velocity error gain - CTG         
xerr_gain = 0.3;    % ACC spacing error gain - CTG          
vx_gain   = 0.5;    % ACC relative velocity gain - CTG
max_acc   = 2;      % Maximum acceleration [m/s^2]
min_acc   = -3;     % Minimum acceleration [m/s^2]



%% PLOTS
directory = "C:\Users\gabri\Documents\TCC - EV powertrain control\Text\images";

% plotting Voc and Roc interpolations
fig = figure();
fig.Position = [100, 100, 1000, 600];
subplot(2,1,1);
plot(SOC_Voc_data(:,1),SOC_Voc_data(:,2),'bo',SOC_vec,Voc_s,'b.');
grid on
title('V_{oc} interpolation');
xlabel('SOC [-]');
ylabel('Open circuit battery voltage [V]');
legend('Sample SOC points','Interpolated values');
subplot(2,1,2);
plot(SOC_Ro_data(:,1),SOC_Ro_data(:,2),'ro',SOC_vec,Ro_s,'r.');
xlabel('SOC [-]');
ylabel('Open circuit battery resistance [\Omega]');
grid on 
title('R_o interpolation');
legend('Sample SOC points','Interpolated values', 'Location','best');

filename = 'battrey_interpolation.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);


% plotting driving cycle velocity and acceleration profiles
fig = figure();
fig.Position = [100, 100, 1000, 600];
subplot(2,1,1)
plot(vehspeed(:,1),vehspeed(:,2),'LineWidth',1.5)
grid on
xlabel('Time [s]');
ylabel('Reference velocity [m/s]');
subplot(2,1,2)
plot(vehspeed(:,1),vehacc,'LineWidth',1.5)
grid on
xlabel('Time [s]');
ylabel('Reference aceleration [m/s^2]');

filename = 'speed_profile.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);


% electric motor efficiency map
fig = figure();
fig.Position = [100, 100, 1000, 600];
levels = [0:0.70:0.70 00.70:0.1:0.94 0.94:0.01:1]*100;
M = contourf(Speed,Shaft_Torque,Efficiency,levels);
hold on
plot(Speed_max,Torque_max)
grid on 
title('EM efficiency map')
xlabel('EM speed [rpm]')
ylabel('Shaft Torque [Nm]')

filename = 'EM_eff_map.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);

% return

%% MPC

% PARAMETERS

t_sim = 400; % simulation time [s]

Ts = 0.05; % Sampling time

par.nx         = 3; % number of states
par.nu         = 1; % control elements number
par.ny         = 3; % number of outputs
par.model = @prediction_longitudinal_model; %Modello di predizione
par.ub         = 250; % Upper bound saturazione input -> maximum value for control output 
par.lb         = -250; % Lower bound saturazione input -> minimum value for control output 
par.tol        = 1; % Reference tolerance
par.Nfev       = 150; % Interation number of fmincon in cost function (default 200)
par.Ts         = Ts;


% initial conditions
SOC0 = 0.8;
x0 = 0;
v0 = vehspeed(1);
xx0 = [SOC0; x0; v0];

return