function [xdot, y] = prediction_longitudinal_model(t,X,U) % add y to output eventually
% arguments: t, X: states,
% U: command in the previous instant

% enviroment parameters
g = 9.81; % Gravity acceleration [m/s^2]
alpha = 0*pi/180; % road slope [rad]
rho = 1.25; % air density [kg/m^3] - from Onori HEV book

% vehicle parameters
M_veh = 1400; % vehicle mass [kg]
a = 1; % Front axle - CoG  Front axle - CoG  [m]
b = 1.3; % Rear axle - CoG  [m]
h = 0.3; % Height of CoG [m]
f_0 = 4.5/1000; % Static rolling coefficient [N/kN]
k = 0; % miscellaneous loss coeff [Ns/m]
r_w = 0.3; % Wheel radius [m]
C_d = 0.33; % Drag coeff
A_f = 2.15; % Frontal area [m^2]
tau_gb = 9.6; % Gear ratio
eta_gb = 0.97; % Gearbox efficiency
eta_inv = 1; % Inverter efficiency
phi = r_w/tau_gb;

% battery parameters
N_s = 108; % Number of series
N_p = 1; % Number of parallels
N_b = N_s*N_p; % number of battery cells
Q_nom = 60*N_p*3600; % nominal battery capacity [As]
eta_batt = 0.95; % Coloumbic efficiency

% state variables
x1 = X(1); % SOC
x2 = X(2); % Position
x3 = X(3); % Velocity

% control input
T_EM = U;

% Forces 
F_roll = M_veh*g*f_0; % rolling resistance force
F_aero = 0.5*C_d*rho*A_f*x3^2; % aerodynamic force

% EM equations
w_EM = x3/phi; 
w_EM_rpm = w_EM*30/pi;
eta_EM = eff_poly(w_EM_rpm, T_EM);
P_EM = w_EM*T_EM;

% battery equations
Pb = P_EM/((eta_EM*eta_inv)^(sign(P_EM)));
V_oc = N_s*V_poly(x1); % vector with interpolated values of Voc_s (single cell)
R_o = N_s/N_p*R_poly(x1); % vector with interpolated values of Ro_s (single cell)
Ib = (V_oc - sqrt(V_oc^2 - 4*R_o*Pb))/(2*R_o);

% state equations
soc_dot = -Ib/(Q_nom*eta_batt^(sign(Ib)));
x_dot = x3;
v_dot = ((eta_gb/phi)*T_EM - F_roll - F_aero)/M_veh;

% state derivative
xdot = [soc_dot;x_dot;v_dot];

y = [x1;x2;x3];
end